
loadfile("scaffold.lua")()
loadfile("terra.lua")()
loadfile("ui.lua")()
loadfile("worldparams.lua")()

local _G = _G -- hmmm

function makegame()

print("=====================")
local world = setmetatable({}, {__index = _G})
print("WORLDA", world, world.complete, _G, _G.complete)
world._G = world
setfenv(1, world)


print("WORLD", world, world.complete, _G, _G.complete)
print("=====================")



local terra = terra_init(mappieces, entities)
terra.load(io.snatch(override_level or "lev.lev.lua"))

local scaffold = scaffold_init()

terra.scaffoldize(scaffold.InsertLine)

xcenter, ycenter = 0, 0
local function pixclamp(x)
  return math.floor(x * 32 + 0.5) / 32
end



local monsters = {}
local dying = {}
local painful = {}
local newpainful = {}
local alwayspainful = {}
local monsterpainful = {}

local player

showhealth = 120

local function collide(one, two)
  return math.abs(two.x - one.x) * 2 <= two.width + one.width and math.abs(two.y - one.y) * 2 <= two.height + one.height
end

function stdrender(self)
  if collide(self, {x = xcenter + 16, y = ycenter + 12, width = 32, height = 24}) then
    --print("render", self.x, self.y, self.width, self.height)
    local bright = self.dead and 0.5 or 1
    local sprit = self.tex
    if sprit.get then sprit = sprit:get() end
    
    local by = self.y + self.height / 2 + (self.tex_voffset or 0)
    
    local widdi = sprit:GetWidth() / 64
    if self.direction then widdi = -widdi * self.direction end
    if self.mirror then widdi = -widdi end
    
    RenderBoundedSprite(sprit, self.x - widdi, by - sprit:GetHeight() / 32, self.x + widdi, by, bright, bright, bright, self.tex_alpha or 1)
  end
end

local function proxivol(x, y)
  local xm = math.abs(x - (xcenter + 8))
  if xm < 16 then xm = 1 elseif xm < 32 then xm = 1 - (xm - 16) / 16 else xm = 0 end
  local ym = math.abs(y - (ycenter + 6))
  if ym < 12 then ym = 1 elseif ym < 24 then ym = 1 - (ym - 12) / 12 else ym = 0 end
  
  return xm * ym
end



local sequences = {
 scout = {len = 6, sloth = 8, step = "walk", steplen = 3},
 scout_backward = {len = 6, sloth = 12, direction = -1, prefix = "scout"},
 scout_death = {len = 2, sloth = 20, norepeat = true},
 scout_slash = {len = 4, sloth = 5, norepeat = true},
 
 commando_death = {len = 2, sloth = 20, norepeat = true},
 commando_flamer = {len = 6, sloth = 5, norepeat = true},
 
 ninja_death = {len = 2, sloth = 20, norepeat = true},
 
 dustblast = {len = 3, sloth = 10, norepeat = true},
 
 kaboom = {len = 3, sloth = 6, norepeat = true},
 
 bear = {len = 4, sloth = 12, step = "walk", steplen = 2},
 bear_cannon = {len = 4, sloth = 12, step = "walk", steplen = 2},
 
 bolt = {len = 3, sloth = 8},
 bolt_impact = {len = 4, sloth = 5, norepeat = true},
 
 slash = {len = 5, sloth = 5},
}
local gtex_reserve = {}
local function gtex(id)
  if not gtex_reserve[id] then gtex_reserve[id] = Texture(id) end
  return gtex_reserve[id]
end
local function std_seq_get(self)
  local itex
  if self.seq.direction == -1 then
    itex = self.seq.len - math.mod(math.floor(self.cpos / self.seq.sloth), self.seq.len)
  else
    itex = math.mod(math.floor(self.cpos / self.seq.sloth), self.seq.len) + 1
  end
  return gtex((self.seq.prefix or self.name) .. "_" .. itex)
end
local function ssg_argh(self)
  return math.mod(math.floor(self.cpos / self.seq.sloth), self.seq.len) + 1
end
local function std_seq_increment(self, x, y)
  local t = ssg_argh(self)
  self.cpos = self.cpos + 1
  local n = ssg_argh(self)
  
  if self.seq.steplen then
    t = math.floor(t / self.seq.steplen)
    n = math.floor(n / self.seq.steplen)
  end
  
  if t ~= n and self.seq.step and x and y then
    local v = proxivol(x, y)
    
    if v ~= 0 then
      PlaySound(self.seq.step, v)
    end
  end
  
  if self.cpos_done and self.cpos >= self.cpos_done then self.cpos = self.cpos_done - 1 end
end
local function std_done(self)
  return self.cpos_done == self.cpos + 1
end
local function NewSequence(id)
  assert(sequences[id], id)
  
  local rv = {seq = sequences[id], name = id}
  if rv.seq.norepeat then
    rv.cpos = 0
    rv.cpos_done = rv.seq.sloth * rv.seq.len
  else
    rv.cpos = math.random(rv.seq.len * rv.seq.sloth)
  end
  rv.get = std_seq_get
  rv.increment = std_seq_increment
  rv.done = std_done
  
  return rv
end


print("WORLD2", world, world.complete)
--[[ #####################################
                                 AI ASSISTANCE
#######################################]]


local monster_scripts = {}

local function PlayerIsWithin(self, dist)
  local pdx = player.x - self.x
  local pdy = player.y - self.y
  local pdd = pdx / math.abs(pdx)
  pdx, pdy = math.abs(pdx), math.abs(pdy)
  
  return pdd == self.direction and pdx < dist and pdy < dist
end
  
--[[ #####################################
                                 SCOUT AI
#######################################]]

do
  local far_alert = 10
  local med_alert = 8
  local close_alert = 4
  
  monster_scripts.scout = {}

  
  local function alert_coro(self)
    self.xvel = 0
    
    self.tex = Texture("scout_crouch")
    local cr = true
    local crt = 0
    -- wait for the player to either get very close, or stay within/without semiclose for a certain number of frames
    
    while crt < 120 do
      if PlayerIsWithin(self, close_alert) then
        cr = true
        break
      end
      
      local tr = PlayerIsWithin(self, med_alert)
      if tr == cr then
        crt = crt + 1
      else
        cr = tr
        crt = 0
      end
      
      coroutine.yield()
    end
    
    if cr == false then
      self.alert_coro = nil
      return
    end
    
    local cpc = "wut"
    while true do
      local pdd = self.x - player.x
      pdd = math.abs(pdd) / pdd
      self.direction = -pdd
      
      local expectipos = player.x + pdd * (player.width / 2 + self.width / 2 + 0.1)
      assert(self.center_x)
      
      if expectipos >= self.center_x - self.prototype.scout_hbounds and expectipos <= self.center_x + self.prototype.scout_hbounds then
        self.xvel = approach(self.x, expectipos, 0.08) - self.x
      else
        expectipos = player.x + pdd * 8
        
        if expectipos < self.center_x - self.prototype.scout_hbounds or expectipos > self.center_x + self.prototype.scout_hbounds then
          self.alert_coro = nil
          return
        end
        
        self.xvel = approach(self.x, expectipos, 0.04) - self.x
      end
      
      if math.abs(self.xvel) < 0.01 then
        if cpc ~= "stand" then
          self.tex = Texture("scout_crouch")
          cpc = "stand"
        end
      else
        if self.xvel * self.direction > 0 then
          if cpc ~= "forward" then
            self.tex = NewSequence("scout")
            cpc = "forward"
          end
        else
          if cpc ~= "backward" then
            self.tex = NewSequence("scout_backward")
            cpc = "backward"
          end
        end
      end
      
      coroutine.yield()
    end
  end
  
  local function swipe_coro(self)
    print("swipestart")
    self.xvel = 0
    
    PlaySound("slash")
    local monst = {x = self.x + self.direction * 1.0, y = self.y - 0.8, width = 0.8, height = 1.6, lifetime = 20, tick = function (self) self.tex:increment(self.x, self.y) self.lifetime = self.lifetime - 1 if self.lifetime <= 0 then dying[self] = true end end, render = stdrender, direction = self.direction, tex = NewSequence("scout_slash")}
    monsters[monst] = true
    painful[monst] = 0.1
    
    coroutine.pause(10)
    
    dying[monst] = true
    
    self.swipe_coro = nil
  end
  
  function monster_scripts.scout:damaged()
    self.direction = math.abs(player.x - self.x) / (player.x - self.x)
  end
  
  function monster_scripts.scout:think()
    if PlayerIsWithin(self, far_alert) and player.y < self.y + 3 and not self.alert_coro then
      print("alerting")
      self.alert_coro = coroutine.wrap(alert_coro, self)
    end
    
    if self.alert_coro then
      if (not self.swipe_coro) and math.abs(player.y - self.y) < 1 and PlayerIsWithin(self, self.width / 2 + player.width / 2 + 0.3) then
        self.swipe_coro = coroutine.wrap(swipe_coro, self)
      end
      
      if self.swipe_coro then
        self.swipe_coro()
      else
        self.alert_coro()
      end
    else
      self.quiet_coro()
    end
  end
  
  function monster_scripts.scout:quiet_coro()
    while true do
      local goalx = self.center_x + math.random() * 2 * self.prototype.scout_hbounds - self.prototype.scout_hbounds
      
      local last = -1000
      while math.abs(goalx - self.x) > 0.01 and math.abs(last - self.x) > 0.01 do
        self.xvel = approach(self.x, goalx, 0.05) - self.x
        self.direction = math.abs(self.xvel) / self.xvel
        last = self.x
        coroutine.yield()
      end
      
      self.xvel = 0
      
      self.tex = Texture("scout_stand")
      coroutine.pause(math.random() * 120 + 60)
      self.tex = NewSequence("scout")
    end
  end
  
  function monster_scripts.scout:death()
    self.tex = NewSequence("scout_death")
  end
  
  function monster_scripts.scout:init()
    self.center_x = self.x
    self.quiet_coro = coroutine.wrap(self.quiet_coro, self)
    
    self.tex = NewSequence("scout")
  end
end
print("WORLD3", world, world.complete)
--[[ #####################################
                                 COMMANDO AI
#######################################]]

do
  monster_scripts.commando = {}
  
  function monster_scripts.commando:coro()
    while true do
      local pdx = player.x - self.x
      self.direction = pdx / math.abs(pdx)
      
      if PlayerIsWithin(self, 18) then
        PlaySound("flame")
        self.tex = Texture("commando_flaming")
        local monst = {x = self.x + (self.prototype.commando_blastlen / 2 + 1) * self.direction, y = self.y + 0.5, width = self.prototype.commando_blastlen, height = 3, life = 30, direction = self.direction,
          tick = function (self)
            self.life = self.life - 1
            if self.life <= 0 then
              dying[self] = true
            end
            self.tex:increment()
          end,
          render = stdrender,
          tex = NewSequence("commando_flamer")}
        monsters[monst] = true
        painful[monst] = 1
        coroutine.pause(15)
        self.tex = Texture("commando_crouch")
        
        coroutine.pause(105)
      end
      
      coroutine.yield()
    end
  end
  
  function monster_scripts.commando:init()
    self.tex = Texture("commando_crouch")
  end
  
  function monster_scripts.commando:death()
    self.tex = NewSequence("commando_death")
  end
end

--[[ #####################################
                                 NINJA AI
#######################################]]

do
  monster_scripts.ninja = {}
  
  monster_scripts.ninja.direction = 1
    
  function monster_scripts.ninja:init()
    self.orig_x, self.orig_y = self.x, self.y
    
    self.tex = Texture("ninja_crouch")
  end
  
  function monster_scripts.ninja:coro()
    local reset = false
    
    local function fade(t, s, e)
      self.tex_alpha = s
      for i = 1, t do
        coroutine.yield()
        self.tex_alpha = (i / t) * (e - s) + s
      end
      self.tex_alpha = e
    end
    while true do
      while not (PlayerIsWithin(self, 12) and player.y > self.y - 4 and player.y < self.y + 8) do
        coroutine.yield()
      end
      print(player.y, self.y, not (PlayerIsWithin(self, 12) and player.y < self.y - 4 and player.y > self.y + 4), player.y < self.y - 4, player.y > self.y + 4)
      
      -- fade out
      PlaySound("ninja_fadeout")
      fade(20, 1, 0)
    
      while true do
        -- don't collide for a bit
        self.x = -1000
        self.y = -1000
        self.gravity = 0
        self.freefall = true
        coroutine.pause(40) -- hidden
        
        while true do -- in ether, trying to attack
          if player.x < self.orig_x or player.x > self.orig_x + self.prototype.ninja_hbounds then reset = true break end -- sigh
          
          self.x, self.y = player.x, player.y - 12
          self.freefall = true
          self.gravity = 0
          
          if scaffold.is_valid_location(self.x, self.y, self.width, self.height) then
            break
          else
            self.x = -1000
            self.y = -1000
            coroutine.yield()
          end
        end
        if reset then reset = false break end
        
        self.tex = Texture("ninja_stabby")
        
        self.tex_voffset = 1.5
        PlaySound("ninja_fadein")
        fade(40, 0, 1)
        
        painful[self] = 1
        self.gravity = 1.5
        
        while self.freefall do
          coroutine.yield()
        end
        
        local dust_height = 2.5
        local dust_width = 6
        
        local function makedust(direction)
          local monst = {x = self.x + direction * dust_width / 4, y = self.y + self.height / 2 - dust_height / 2, width = dust_width / 2, height = dust_height, life = 30, direction = direction,
          tick = function (self)
            self.life = self.life - 1
            if self.life <= 0 then
              dying[self] = true
            end
            self.tex:increment()
          end,
          render = stdrender,
          tex = NewSequence("dustblast")}
          
          monsters[monst] = true
          painful[monst] = painful[self]
        end
        PlaySound("ninja_stab")
        
        makedust(-1)
        makedust(1)
        
        painful[self] = nil
        
        coroutine.pause(60) -- boooom
        
        PlaySound("ninja_fadeout")
        fade(20, 1, 0)
        
        self.tex = Texture("ninja_crouch")
        self.tex_voffset = nil
      end
      
      -- fade in
      self.x, self.y = self.orig_x, self.orig_y
      --PlaySound("ninja_fadein") -- ninja is quiet
      fade(20, 0, 1)
    end
  end
  
  function monster_scripts.ninja:death()
    self.tex = NewSequence("ninja_death")
    self.tex_voffset = nil
    self.tex_alpha = nil
  end
end
print("WORLD4", world, world.complete)
--[[ #####################################
                         EXPLOSIVE AI
#######################################]]

local function Kaboom(x, y)
  PlaySound("explosion", proxivol(x, y))
  local kaboom = {
    x = x, y = y, width = 4, height = 4, direction = -1, tex = NewSequence("kaboom"),
    tick = function (self)
      self.tex:increment()
      --print(self.tex.cpos_done, self.tex.cpos)
      if self.tex:done() then dying[self] = true end
    end,
    render = stdrender,
  }
  monsters[kaboom] = true
end

local function comboplosion(targ)
  local combo
  combo = {
    tick = coroutine.wrap(function (self)
      for i = 1, 10 do
        Kaboom(targ.x + math.random(targ.width) - targ.width / 2, targ.y + math.random(targ.height) - targ.height / 2)
        coroutine.pause(4)
      end
      
      dying[combo] = true
    end),
    render = function () end,
  }
  
  monsters[combo] = true
end

local function MakeExplosive(x, y, freefall)
  local rocket = scaffold.Entity(x, y)
  rocket.freefall = true
  rocket.gravity = freefall and 1 or 0
  rocket.width = 1
  rocket.height = 1
  
  function rocket:tick()
    local cx, cy = self.xvel, self.yvel
    
    self:default_tick()
    
    if freefall then
      if self.xvel ~= cx or self.yvel == 0 then
        self:impact()
      end
    else
      if self.xvel ~= cx or self.yvel ~= cy then
        self:impact()
      end
    end
  end
  
  function rocket:impact()
    dying[self] = true
    local blast
    blast = {x = self.x, y = self.y, width = 4, height = 4, direction = -1,
      tick = coroutine.wrap(function ()
        coroutine.yield()
        dying[blast] = true
        coroutine.yield()
      end),
      render = function () end}
    monsters[blast] = true
    newpainful[blast] = 1
    
    Kaboom(self.x, self.y)
  end
  
  return rocket
end

--[[ #####################################
                         ROCKET BOSS AI
#######################################]]

do
  local targets = {}

  monster_scripts.boss_rocket_target = {}
  function monster_scripts.boss_rocket_target:tick() -- we do absolutely nothing
  end
  function monster_scripts.boss_rocket_target:render() -- we do absolutely nothing
  end
  function monster_scripts.boss_rocket_target:init()
    table.insert(targets, {x = self.x, y = self.y})
  end
  
  local fptx, fpty = -2, -0.7
  
  monster_scripts.boss_rocket = {}
  function monster_scripts.boss_rocket:coro()
    -- we do init here so we can guarantee the targets all exist
    local function ang(x)
      return math.atan2(x.x - self.x - fptx, x.y - self.y - fpty)
    end
    table.sort(targets, function(a, b) return ang(a) > ang(b) end)
    
    local ctarg = 1
    
    while true do
      local rocket = MakeExplosive(self.x + fptx, self.y + fpty, false)
      rocket.width = 1
      rocket.height = 1
      rocket.tex = Texture("orb")
      rocket.render = stdrender
      local x, y = targets[ctarg].x - self.x - fptx, targets[ctarg].y - self.y - fpty
      local len = math.sqrt(x*x + y*y)
      x, y = x / len, y / len
      
      rocket.xvel, rocket.yvel = x * 0.5, y * 0.5
      
      monsters[rocket] = true
      painful[rocket] = 0
      
      self.tex = Texture("boss_fire")
      coroutine.pause(5)
      self.tex = Texture("boss_stand")
      coroutine.pause(15)
      
      ctarg = ctarg + 1
      if ctarg > #targets then ctarg = 1 end
    end
  end
  
  function monster_scripts.boss_rocket:death()
    alwayspainful[self] = nil
    painful[self] = nil
    comboplosion(self)
  end
  
  function monster_scripts.boss_rocket:init()
    self.tex = Texture("boss_stand")
  end
end

--[[ #####################################
                         GRENADE BOSS AI
#######################################]]

do
  local fptx, fpty = -2, -0.7
  
  monster_scripts.boss_grenade = {}
  function monster_scripts.boss_grenade:coro()
    while true do
      local rocket = MakeExplosive(self.x + fptx, self.y + fpty, true)
      rocket.width = 1
      rocket.height = 1
      rocket.gravity = 0.4
      rocket.tex = Texture("orb")
      rocket.render = stdrender
      rocket.yvel = -math.random() / 2 + 0.1
      rocket.xvel = -math.random() * 0.2 - 0.05
      
      monsters[rocket] = true
      painful[rocket] = 0
      
      self.tex = Texture("boss_fire")
      coroutine.pause(5)
      self.tex = Texture("boss_stand")
      coroutine.pause(40)
    end
  end
  
  function monster_scripts.boss_grenade:death()
    alwayspainful[self] = nil
    painful[self] = nil
    comboplosion(self)
  end
  
  function monster_scripts.boss_grenade:init()
    self.tex = Texture("boss_stand")
  end
end

--[[ #####################################
                                RIG AI
#######################################]]
print("WORLD5", world, world.complete)
do
  monster_scripts.rig_endgame = {}
  function monster_scripts.rig_endgame:render()
    -- lul
  end
  function monster_scripts.rig_endgame:tick()
    if collide(self, player) then
      complete = "win"
    end
  end
end

--[[ #####################################
                                 END AI
#######################################]]

local function corpsify(target)
  target.bouncefactor = 0.55
  target.bouncelimit = 0.01
  target.gravity = 1
  target.freefall = true
  target.dead = true
  
  target.think = function ()
    if not target.freefall then
      target.xvel = approach(target.xvel, 0, 0.01)
    end
  end
  target.coro = nil
end

function NewMonster(monster)
  local ent = scaffold.Entity(monster.x, monster.y)
  
  ent.tex = Texture(entities[monster.id].tex)
  ent.width, ent.height = entities[monster.id].width, entities[monster.id].height

  if not entities[monster.id].flying and not entities[monster.id].not_real then
    scaffold.ground_anchor(ent)
  end
  
  ent.render = stdrender
  ent.prototype = entities[monster.id]
  ent.damageable = ent.prototype.damageable
  ent.health = ent.prototype.health
  
  if monster_scripts[monster.id] then
    for k, v in pairs(monster_scripts[monster.id]) do
      ent[k] = v
    end
  end
  if ent.coro then ent.coro = coroutine.wrap(ent.coro, ent) end
  
  if ent.init then ent:init() end
  
  ent.takedamage = function(self, amount)
    self.health = self.health - amount
    if self.health <= 0 then
      self.damageable = false
      corpsify(self)
      print("death1")
      if self.death then self:death() end
      PlaySound("death")
    end
  end
  
  if ent.prototype.toxic then
    alwayspainful[ent] = ent.prototype.toxic
  end
  
  return ent
end

for _, v in pairs(terra.entities()) do
  local monst = NewMonster(v)
  
  if entities[v.id].player then
    if override_x then
      local np = {}
      for k, v in pairs(v) do
        np[k] = v
      end
      np.x, np.y = override_x, override_y
      xcenter, ycenter = override_x - 16, override_y - 12
      monst = NewMonster(np)
    end
    
    player = monst
  else
    monsters[monst] = true
  end
end


local bullet_life

local swiping = {x = 0, y = 0, width = 2.5, height = 1.5, tick = function (self) self.tex:increment(self.x, self.y) end, render = stdrender, damage = 0.2, tex = NewSequence("slash")} -- Slashy slashy!

local walk_top_speed = 0.1
local run_top_speed = 0.1
local jump_top_speed = 0.1
local start_accel = 0.0075
local stop_accel = 0.0075
local stop_accel_air = 0.0040
local jump_yvel = 0.25

local jumplift_grav = 0.6
print("WORLD6", world, world.complete)
player.stun = 0
player.mirror = true
player.direction = 1
function player:mover(x, y, jumpy)
  if self.invincibility == true then
    if not self.freefall then
      self.stun = 30
      self.invincibility = 60
    else
      x, y, jumpy = 0, 0, false
    end
  end
  
  if self.dead or self.stun > 0 then x = 0 y = 0 jumpy = false end
  
  if x ~= 0 then
    if self.state == "stopped" then
      self.state = "running"
    end
    
    self.direction = x / math.abs(x)
  elseif self.invincibility ~= true then
    local stop_acel = (self.freefall and stop_accel_air or stop_accel)
    if stop_acel > math.abs(self.xvel) then
      self.xvel = 0
      if not self.freefall then self.state = "stopped" end
    else
      if self.xvel > 0 then stop_acel = -stop_acel end
      self.xvel = self.xvel + stop_acel
    end
  end

  if y < 0 and not self.freefall and jumpy then
    self.freefall = true
    self.gravity = jumplift_grav
    self.yvel = -jump_yvel
    self.state = "run_jump"
  end
  
  if y >= 0 and self.freefall and self.gravity ~= 1 then
    self.yvel = self.yvel / 2
    self.gravity = 1
  end
  
  if self.state == "run_jump" and self.yvel > 0 then
    self.state = "run_jump_fall"
    self.gravity = 1
  end
  
  if x > 0 then
    self.xacel = start_accel
  elseif x < 0 then
    self.xacel = -start_accel
  else
    self.xacel = 0
  end
  
  if self.xacel * self.xvel < 0 then
    self.xacel = self.xacel * 2
  end
  
  if self.invincibility ~= true then
    local top_speed
    if self.state == "stopped" or self.state == "end_run_jump" then
      top_speed = walk_top_speed
    elseif self.state == "running" then
      top_speed = run_top_speed
    elseif self.state == "run_jump" or self.state == "run_jump_fall" or self.state == "start_run_jump" then
      top_speed = jump_top_speed
    else
      top_speed = 1000
    end
    
    local expected_speed = self.xvel + self.xacel
    --print("speedz", expected_speed, top_speed)
    if math.abs(expected_speed) > top_speed then
      --print("top flattening", expected_speed, top_speed, (top_speed - expected_speed) * (expected_speed / math.abs(expected_speed)))
      self.xacel = (top_speed - math.abs(expected_speed)) * (expected_speed / math.abs(expected_speed))
    end
  end
  
  local img
  if self.dead then
    img = "_corpse"
  elseif self.freefall then
    img = "_jump"
  elseif self.xvel == 0 then
    img = "_stand"
  else
    img = ""
  end
  
  local prefix = "bear"
  if bullet_life > 0 then
    prefix = "bear_cannon"  -- bearcannon
  end
  
  if self.current_tex ~= img then
    self.current_tex = img
    if img == "" then
      self.tex = NewSequence(prefix)
    else
      self.tex = Texture(prefix .. img)
    end
  end
  
  --if self.tex.increment then self.tex:increment(self.x, self.y) end
  
  if type(self.invincibility) == "number" then
    self.invincibility = self.invincibility - 1
    if self.invincibility <= 0 then
      self.invincibility = nil
    end
  end
  self.stun = self.stun - 1
end

function upgrade_nanok(legs, armor, blaster)
  walk_top_speed = walk_top_speed * (legs / 2 + 1)
  run_top_speed = run_top_speed * (legs / 2 + 1)
  jump_top_speed = jump_top_speed * (legs / 2 + 1)
  start_accel = start_accel * (legs + 1)
  stop_accel = stop_accel * (legs + 1)
  stop_accel_air = stop_accel_air * (legs + 1)
  jump_yvel = jump_yvel * (legs / 4 + 1)
  player.health_max = 3 * (armor + 1) - 0.01
  player.health = player.health_max
  bullet_life = blaster * 20
end

--upgrade_nanok(0, 2, 0)
--upgrade_nanok(1, 1, 1, 1)
--upgrade_nanok(2, 2, 2)
--upgrade_nanok(2, 2, 2)



local function focusize(oldcenter, focuspoint, slack)
  if math.abs(oldcenter - focuspoint) > slack then
    oldcenter = approach(oldcenter, focuspoint, math.max(0.03, math.abs(oldcenter - focuspoint) / 40))
  end
  return oldcenter
end

print("WORLD7", world, world.complete)

local function impact(x, y)
  local kaboom = {
    x = x, y = y, width = 0.5, height = 0.5, direction = -1, tex = NewSequence("bolt_impact"),
    tick = function (self)
      self.tex:increment()
      --print(self.tex.cpos_done, self.tex.cpos)
      if self.tex:done() then dying[self] = true end
    end,
    render = stdrender,
  }
  monsters[kaboom] = true
end

function tick_loop()
  collectgarbage("stop")
  
  local dx, dy = 0, 0
  local speed = 1
  
  if IsKeyDownFrame("z") then dy = dy - speed end
  if IsKeyDownFrame("arrow_left") then dx = dx - speed end
  if IsKeyDownFrame("arrow_right") then dx = dx + speed end
  
  player:mover(dx, dy, WasKeyPressed("z"))
  player:tick()
  perfbar(1, 0, 0, function ()
    for k in pairs(monsters) do
      k:tick()
    end
  end)
  
  for k in pairs(dying) do
    monsters[k] = nil
    painful[k] = nil
    monsterpainful[k] = nil
  end
  dying = {}
  
  for k, v in pairs(alwayspainful) do
    if not k.dead then
      painful[k] = v
    end
  end
  for k, v in pairs(painful) do
    if collide(player, k) then
      if k.impact then k:impact() end
      
      local ph = player.health
      if not player.invincibility then
        player.health = player.health - v
      end
      
      painful[k] = 0
      
      if math.ceil(ph) ~= math.ceil(player.health) then
        print("punt", ph, player.health)
        player.health = math.ceil(player.health) - 0.01
        player.invincibility = true
        player.freefall = true
        player.gravity = 1
        player.yvel = -0.15
        player.xvel = 0.4 * k.direction
        
        showhealth = 120
        
        PlaySound("beardeath")
        
        if player.health <= 0 and not player.dead then
          player.dead = 120
        end
      end
    end
  end
  for k, v in pairs(newpainful) do
    painful[k] = v
  end
  newpainful = {}
  
  perfbar(1, 0, 1, function ()
    for m in pairs(monsters) do
      if m.damageable then
        for p in pairs(monsterpainful) do
          if collide(m, p) then
            m:takedamage(p.damage)
            if m.damaged then m:damaged() end
            
            if p.damage_singlehit then
              dying[p] = true
              impact(p.x - p.direction * p.width / 2, p.y)
            end
          end
        end
      end
    end
  end)
  
  if not player.dead and player.stun <= 0 and IsKeyDownFrame("c") then
    monsters[swiping] = true
    monsterpainful[swiping] = true
    sequences.slash.step = "slash"
  else
    monsters[swiping] = nil
    monsterpainful[swiping] = nil
    sequences.slash.step = nil
  end
  
  if not player.dead and player.stun <= 0 and WasKeyPressed("x") and bullet_life > 0 then
    PlaySound("shoot")
    
    local bullet = NewMonster{x = player.x + player.direction * 0.5, y = player.y - 1.5625, id = "bullet"}
    bullet.freefall = true
    bullet.gravity = 0
    bullet.damage = 1
    bullet.direction = -player.direction
    bullet.damage_singlehit = true
    bullet.xvel_orig = player.direction * 0.5
    bullet.xvel = bullet.xvel_orig
    bullet.lifetime = bullet_life
    bullet.tex = NewSequence("bolt")
    function bullet:tick()
      self.lifetime = self.lifetime - 1
      if self.lifetime <= 0 then
        dying[self] = true
      end
      
      self:default_tick()
      
      if self.xvel ~= self.xvel_orig then
        dying[self] = true  -- exploooooode
        impact(self.x - self.direction * self.width / 2, self.y)
      end
      
      self.tex:increment()
    end
    print("validated", scaffold.is_valid_location(bullet.x, bullet.y, bullet.width, bullet.height))
    if scaffold.is_valid_location(bullet.x, bullet.y, bullet.width, bullet.height) then
      monsterpainful[bullet] = true
      monsters[bullet] = true
    else
      impact(bullet.x, bullet.y)
    end
  end
  
  swiping.x = player.x + player.direction * 1.5
  swiping.y = player.y
  
  xcenter = focusize(xcenter, player.x + player.direction * 5 - 16, 4)
  ycenter = focusize(ycenter, player.y - 12, 4)
  
  showhealth = showhealth - 1
  
  if player.dead then player.dead = player.dead - 1 end
  if player.dead == 0 then complete = "lose" print("setting lose from player", player) end
end
local bg = Texture("bg")
function render(...)
  gl.Disable("CULL_FACE")
  
  RenderBoundedSprite(bg, 0, 0, 1024, 768)
  
  gl.MatrixMode("PROJECTION")
  gl.LoadIdentity()
  gl.Ortho(0, 1024, 768, 0, -1, 1)
  
  gl.MatrixMode("MODELVIEW")
  gl.LoadIdentity()
  
  gl.MultMatrix{32, 0, 0, 0,    0, 32, 0, 0,    0, 0, 32, 0,    0, 0, 0, 1}
  
  local lxc, lyc = pixclamp(xcenter), pixclamp(ycenter)
  gl.Translate(-lxc, -lyc, 0)
  
  perfbar(0.5, 0.5, 0.5, function ()
    terra.render_buckets(math.floor(lxc), math.floor(lyc), math.ceil(lxc) + 32, math.ceil(lyc) + 24)
    --terra.render(math.floor(lxc), math.floor(lyc), math.ceil(lxc) + 32, math.ceil(lyc) + 24)
  end)
  
  perfbar(1.0, 0, 0, function ()
    player:render()
    for v, _ in pairs(monsters) do
      v:render()
    end
  end)
  
  if showhealth >= 0 then
    RenderBoundedBox(1, 0, 0, player.x - player.width / 2, player.y - 2, player.x + player.width / 2, player.y - 1.8)
    RenderBoundedBox(1, 1, 1, player.x - player.width / 2, player.y - 2, player.x - player.width / 2 + player.width * player.health / player.health_max, player.y - 1.8)
  end
  
  -- reset to standard glop
  gl.MatrixMode("PROJECTION")
  gl.LoadIdentity()
  gl.Ortho(0, 1024, 768, 0, -1, 1)
  
  gl.MatrixMode("MODELVIEW")
  gl.LoadIdentity()
  
  perfbar(0, 0, 1, function ()
    UIParent:Render()
  end)
end
print("WORLD7", world, world.complete)
return world

end







-- think there's more

local game

local diatex
local displaytex = Texture("nanokhome_1")

local tbocks = CreateFrame("Frame")
tbocks:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", 100, -100)
tbocks:SetPoint("RIGHT", UIParent, "RIGHT", -100)
tbocks:SetHeight(150)

local display = CreateFrame("Frame", tbocks)
display:SetPoint("TOPLEFT", tbocks, "TOPLEFT")
display:SetWidth(tbocks:GetHeight())
display:SetHeight(tbocks:GetHeight())
function display:Draw()
  if diatex then RenderBoundedSprite(diatex, self:GetBounds()) end
end

local werdbox = CreateFrame("Frame", tbocks)
werdbox:SetPoint("TOPLEFT", display, "TOPRIGHT", 30, 0)
werdbox:SetPoint("BOTTOMRIGHT", tbocks, "BOTTOMRIGHT")
werdbox:SetBackgroundColor(0, 0, 0)

local werds = CreateFrame("Text_Multiline", werdbox)
werds:SetPoint("TOPLEFT", werdbox, "TOPLEFT", 10, 10)
werds:SetPoint("BOTTOMRIGHT", werdbox, "BOTTOMRIGHT", -10, -10)

local bghammer = CreateFrame("Frame", tbocks)
bghammer:SetPoint("TOPLEFT", UIParent, "TOPLEFT")
bghammer:SetPoint("RIGHT", UIParent, "RIGHT")
bghammer:SetPoint("BOTTOM", tbocks, "TOP")

local bgcenter = CreateFrame("Frame", bghammer)
bgcenter:SetPoint("CENTER", bghammer, "CENTER")
bgcenter:SetHeight(361)
bgcenter:SetWidth(500)
function bgcenter:Draw()
  if displaytex then RenderBoundedSprite(displaytex, self:GetBounds()) end
end

local options = CreateFrame("Frame")

local opts = {}

local function makeopt(click)
  local bigbound = CreateFrame("Frame", options)
  if #opts > 0 then
    bigbound:SetPoint("TOP", opts[#opts], "BOTTOM")
  else
    bigbound:SetPoint("TOP", UIParent, "TOP")
  end
  
  bigbound:SetPoint("LEFT", UIParent, "LEFT")
  bigbound:SetPoint("RIGHT", UIParent, "RIGHT")
  
  bigbound:SetHeight(UIParent:GetHeight() / 4)
  
  local innerbound = CreateFrame("Frame", bigbound)
  innerbound:SetPoint("CENTER", bigbound, "CENTER")
  innerbound:SetHeight(130)
  innerbound:SetWidth(800)
  innerbound:SetBackgroundColor(0, 0, 0)
  
  local highlightbound = CreateFrame("Button", bigbound)
  highlightbound:SetPoint("TOPLEFT", innerbound, "TOPLEFT", -10, -10)
  highlightbound:SetPoint("BOTTOMRIGHT", innerbound, "BOTTOMRIGHT", 10, 10)
  
  innerbound:SetParent(highlightbound)
  
  function bigbound:highlight(itis)
    if itis then
      highlightbound:SetBackgroundColor(1, 0.5, 0.5)
    else
      highlightbound:SetBackgroundColor(0.3, 0.3, 0.3)
    end
  end
  
  local midb = 0.7
  local text1 = CreateFrame("Text", innerbound)
  text1:SetPoint(0.5, 0, innerbound, 0.5, 0, 0, 10)
  text1:SetColor(midb, midb, midb)
  
  local text3 = CreateFrame("Text", innerbound)
  text3:SetPoint(0.5, 1, innerbound, 0.5, 1, 0, -10)
  text3:SetColor(midb, midb, midb)
  
  local text2 = CreateFrame("Text", innerbound)
  text2:SetPoint(0.5, 1, text3, 0.5, 0, 0, -10)
  text2:SetColor(1.0, 1.0, 1.0)
  
  bigbound:highlight(false)
  
  function highlightbound:Click()
    click()
  end
  function bigbound:Activate()
    click()
  end
  function bigbound:SetTopText(txt)
    text1:SetText(txt)
  end
  function bigbound:SetMidText(txt)
    text2:SetText(txt)
  end
  function bigbound:SetBottomText(txt)
    text3:SetText(txt)
  end
  
    
  table.insert(opts, bigbound)
end

local upgrade = {
  {"leg", {"your own four paws", "Spring Shoes", "Turbo Jumpers"}, {"squeaky duck", "rubber ball"}},
  {"armor", {"bear fur", "Beargarb of Protection", "Cladding of the Ursine Champion"}, {"sequined shirt", "pants with bells"}},
  {"weapon", {"None!", "Plasma Blaster", "Devastation Cannon"}, {"food dish", "water dish"}},
}

local upl = {1, 1, 1}

local function settext(targ, cat, lev)
  if lev ~= 3 then
    targ:SetTopText(("Current %s upgrade: %s"):format(upgrade[cat][1], upgrade[cat][2][lev]))
    targ:SetMidText(("Construct %s"):format(upgrade[cat][2][lev + 1]))
    targ:SetBottomText(("(will destroy %s)"):format(upgrade[cat][3][lev]))
  else
    targ:SetTopText(("Current %s upgrade: %s"):format(upgrade[cat][1], upgrade[cat][2][lev]))
    targ:SetMidText("Upgrades complete!")
    targ:SetBottomText("")
  end
end

local function upgrade(targ)
  if upl[targ] == 3 then return end
  
  PlaySound("forge")
  upl[targ] = upl[targ] + 1
  settext(opts[targ], targ, upl[targ])
  
  if upl[1] == 3 and upl[2] == 3 and upl[3] == 3 then
    opts[4]:SetBottomText("(Fully upgraded! Let's do this thing!)")
  end
end

local commence = false

makeopt(function () upgrade(1) end)
makeopt(function () upgrade(2) end)
makeopt(function () upgrade(3) end)
makeopt(function () commence = true end)

local function textreset()
  settext(opts[1], 1, 1)
  settext(opts[2], 2, 1)
  settext(opts[3], 3, 1)

  opts[4]:SetMidText("Assault the alien mothership!")
  opts[4]:SetBottomText("(Sure you're ready? The aliens won't go down without a fight!)")
end
textreset()

options:Hide()

function ShowText(speaker, text)
  werds:SetText(text)
  diatex = Texture(speaker)
  
  coroutine.yield()
  while not WasKeyPressed("z") do
    coroutine.yield()
  end
  
  PlaySound("chat")
end

function ShowConvo(list)
  for _, t in ipairs(list) do
    local prefx = t:sub(1, 5)
    local speak
    
    if prefx == "\"Nano" then
      speak = "ringmaster"
    elseif prefx == "Nanok" then
      speak = "nanok"
    elseif prefx == "\"HEY!" then
      speak = "sprite"
    end
    
    ShowText(speak, t)
  end
end

function RunGame(frames)
  tbocks:Hide()
  
  
  while not game.complete do
    game:tick_loop()
    coroutine.yield()
  end
  local rv = game.complete
  game = nil
  
  tbocks:Show()
  
  return game
end

function gamelogic()
  if not skipitall then
    ShowConvo({
      [["Nanok! Frabjous work today! The audience was gleeful and exuberant. I leave you here for the noctum, in your den of recuperative solitude, which is full of expensive toys, food, and costumes. If any being should wish to communicate with you, merely hit Z to advance to the next page!"]],
      [[Nanok rested, luxurious in the satisfaction of a job well done. Truly, he was a circus bear who enjoyed his job. But sometimes, he thought . . . what . . . if he could be . . . something more?]],
    })
    
    displaytex = Texture("nanokhome_2")
    
    ShowConvo({
      [["HEY! LISTEN!"]],
      [[Nanok woke with surprise! Was someone talking to him?]],
      [["HEY! LISTEN! We need your help!"]],
      [[Nanok regarded the winged orb in confusion. What could he be needed for, besides conventional cavorting and japery?]],
      [["HEY! LISTEN! The planet is under attack by aliens! You have to go shut down their reactor core!"]],
      [[Nanok shook his head sadly, for he was merely a circus bear. What good could a circus bear do against fearsomely armed aliens, as such were nigh-guaranteed to be?]],
      [[Nanok tried to explain his misgivings, though since he didn't speak English, this mostly came across as a loud barking noise.]],
      [["HEY! LISTEN! You're not even trying! You can turn your costumes into armor, and your toys into spring-filled shoes, and your water and food into blaster cannons!"]],
      [[Nanok admitted (again, via barking) that this was an excellent and unarguable point, but those were things the Ringmaster had purchased for him. Could he risk the Ringmaster's wrath by ruining his riches for such a ridiculous reason?]],
      [["HEY! LISTEN! If the planet gets blown up, so will all your toys!"]],
      [[Nanok acquiesced glumly, and pledged to convert the minimum of his possessions into combat equipment. He took stock of his gear.]],
    })
  end
  
  displaytex = nil
  
  local showed_instructions = false
  
  while true do
    --do break end
    
    tbocks:Hide()
    options:Show()
    ShowMouseCursor(true)
    
    local active = 1
    for _, v in ipairs(opts) do
      v:highlight(false)
    end
    
    if not skipitall then
      coroutine.yield()
      while not commence do
        opts[active]:highlight(false)
        if WasKeyPressed("z") then opts[active]:Activate() end
        if WasKeyPressed("arrow_up") then active = active - 1 end
        if WasKeyPressed("arrow_down") then active = active + 1 end
        if active < 1 then active = active + #opts end
        if active > #opts then active = active - #opts end
        opts[active]:highlight(true)
        coroutine.yield()
      end
      commence = false
    else
      upl[1] = 3
      upl[2] = 3
      upl[3] = 3
    end
    
    ShowMouseCursor(false)
    options:Hide()
  
    game = makegame()
    collectgarbage("collect")
    print("compi", game, game.complete, complete)
    assert(not game.complete)
    game.upgrade_nanok(upl[1] - 1, upl[2] - 1, upl[3] - 1)
    if not showed_instructions then
      tbocks:Show()
      ShowConvo({
        [["HEY! LISTEN! Do you remember how to fire your blaster cannon? Or use your claws? Or walk? Or breathe? Do you remember how to breathe?"]],
        [[Nanok found this rather insulting. Very few circus bears ever forgot how to breathe. Statistically, it was insignificant.]],
        [["HEY! LISTEN! Use the arrow keys to move! Push Z to jump! Push C to swipe with your claws! Push X to fire your blaster cannon! You have a blaster cannon right? I wasn't paying attention 'cause I don't really care but you should or you're going to die!"]],
      })
      showed_instructions = true
      tbocks:Hide()
    end
    
    coroutine.yield()
    while not game.complete do
      game:tick_loop()
      coroutine.yield()
    end
    local rv = game.complete
    
    if rv == "win" then
      tbocks:Show()
      ShowConvo({
        [[Nanok found himself plunged into the center of the highly radioactive reactor core, but feared not, as circus bears are immune to radiation. He quickly set about disabling the alien power source, secure in his knowledge that doing so would result in victory.]],
        [["HEY! LISTEN! Good job! The world isn't ruined! I'll teleport you out of the reactor core immediately! ZAPPO!"]],
      })
    end
    
    game = nil
    
    tbocks:Show()
    
    if rv == "lose" then
      ShowConvo({[["HEY! LISTEN! That wasn't very good at all! Let's get you some different equipment!"]]})
    end
    
    if rv == "win" then break end
  end
  
  displaytex = Texture("nanokhome_2")
  
  ShowConvo({
    [[Nanok questioned the sprite, as it seems that if he could be teleported out of a reactor core, he could have been teleported in as well.]],
    [["HEY! LISTEN! Shut up!"]],
    [[Nanok watched as the sprite flitted away, leaving him alone in his den. Several minutes later, he heard a knocking on his door.]],
  })
  
  displaytex = Texture("nanokhome_1")

  local upgtotal = upl[1] + upl[2] + upl[3] - 3
  
  local finaltext
  if upgtotal == 0 then
    ShowConvo({
      [["Nanok! I have come bearing caviar and steak, to celebrate our glorious wealth and our unsurprising, yet somehow notable, lack of radioactive toys. Feast with me, and on the morrow we vacation . . . to the moon!"]],
      [[Nanok barked loudly, excited for his lunar adventure. There would be no aliens on the moon . . . and that would be thanks to him.]]
    })
    finaltext = "EXCELSIOR!"
  elseif upgtotal <= 2 then
    ShowConvo({
      [["Nanok! How are you doing? Life is excellent! Oh, but look at this thing of yours, it is radioactive. Nanok, why is this radioactive? What has happened? It shall have to be replaced. At least we are rich, thanks to your excellent performance. I shall return on the morrow."]],
      [[Nanok rested in his comfortable den, knowing that he was, indeed, a bear to be reckoned with.]]
    })
    finaltext = "MILDLY ABASHED, BUT BADASS"
  elseif upgtotal <= 5 then
    ShowConvo({
      [["Nanok! Your possessions! Why are they damaged and modified and radioactive? I do not understand, but at least it did not affect all of them. Times will be tight, but we shall persevere. Ahoy!"]],
      [[Nanok went back to work, with a heavy heart, but pride for his unsung heroism.]],
    })
    finaltext = "BARELY ACCEPTABLY POWERFUL"
  else  -- we;lp
    ShowConvo({
      [["Nanok! Your possessions! What have you done to them? They are all ruined, and somehow, radioactive! You are the worst circus bear, and you will have to work long hours to replace them."]],
      [[Nanok was shamed, and admitted his fault.]],
      [["Nanok! Stop barking at me and go to work!"]],
    })
    finaltext = "TECHNICALLY, VICTORIOUS"
  end

  totallydone = true
  
  local dimmer = CreateFrame("Frame")
  dimmer:SetPoint("TOPLEFT", UIParent, "TOPLEFT")
  dimmer:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT")
  dimmer:SetLayer(1)
  dimmer:SetBackgroundColor(0, 0, 0, 0.9)
  
  local barf = CreateFrame("Frame", dimmer)
  barf:SetPoint("CENTER", UIParent, "CENTER")
  
  barf:SetWidth(300)
  barf:SetHeight(200)
  
  local yay = CreateFrame("Text", barf)
  yay:SetPoint(0.5, 1, barf, 0.5, 0, 0, -80)
  yay:SetText("Final Score")
  yay:SetColor(1, 1, 1)
  
  local function makebar(intext, n, d)
    local container = CreateFrame("Frame", barf)
    container:SetPoint("LEFT", barf, "LEFT")
    container:SetPoint("RIGHT", barf, "RIGHT")
    
    local text = CreateFrame("Text", container)
    text:SetPoint(1, 0.5, container, 0, 0.5, -20, 0)
    text:SetText(intext)
    text:SetColor(1, 1, 1)
    
    local perc = CreateFrame("Text", container)
    perc:SetPoint(0, 0.5, container, 1, 0.5, 20, 0)
    perc:SetText(n .. " / " .. d)
    perc:SetColor(1, 1, 1)
    
    local bars = CreateFrame("Frame", container)
    bars:SetPoint("TOPLEFT", container, "TOPLEFT")
    bars:SetPoint("BOTTOM", container, "BOTTOM")
    bars:SetPoint(1, nil, container, (n / d), nil)
    bars:SetBackgroundColor(1, 1, 0.8)
    
    return container
  end
  
  local legs = makebar("Intact Toys", 3 - upl[1], 2)
  legs:SetPoint("TOP", barf, "TOP")
  legs:SetHeight(40)
  
  local armor = makebar("Intact Costumes", 3 - upl[2], 2)
  armor:SetPoint("TOP", legs, "BOTTOM", nil, 20)
  armor:SetHeight(40)
  
  local weapon = makebar("Intact Dishes", 3 - upl[3], 2)
  weapon:SetPoint("TOP", armor, "BOTTOM", nil, 20)
  weapon:SetHeight(40)
  
  local total = makebar("Total", 9 - (upl[1] + upl[2] + upl[3]), 6)
  total:SetPoint("TOP", barf, "BOTTOM", nil, 20)
  total:SetHeight(40)
  
  
  local ftext = "VERDICT: " .. finaltext
  local arghtext = CreateFrame("Text", barf)
  arghtext:SetText(ftext)
  arghtext:Hide()
  
  local ftext = CreateFrame("Text_Multiline", barf)
  ftext:SetText("\1S2\1VERDICT: " .. finaltext)
  ftext:SetPoint(0.5, 0, total, 0.5, 1, 0, 80)
  ftext:SetWidth(arghtext:GetWidth() * 2 + 10)
  ftext:SetHeight(100)
  
  
  yay:Hide()
  legs:Hide()
  armor:Hide()
  weapon:Hide()
  total:Hide()
  ftext:Hide()
  
  coroutine.pause(45)
  yay:Show()
  coroutine.pause(45)
  legs:Show()
  coroutine.pause(20)
  armor:Show()
  coroutine.pause(20)
  weapon:Show()
  coroutine.pause(45)
  total:Show()
  coroutine.pause(75)
  ftext:Show()
  
  coroutine.pause(180)
  
  local finaltext = CreateFrame("Text", barf)
  finaltext:SetPoint(0.5, 1, UIParent, 0.5, 1)
  finaltext:SetText("Press Z to restart")
  finaltext:SetColor(1, 1, 1)
  
  while not WasKeyPressed("z") do coroutine.yield() end
  coroutine.yield()
  
  dimmer:Hide()
  upl = {1, 1, 1}
  textreset()
end

local gameit = coroutine.wrap(function () while true do gamelogic() end end)

function tick_loop()
  gameit()
  --return game.tick_loop()
end

function render()
  if game then
    game.render()
  end
  
  UIParent:Render()
end


ShowMouseCursor(false)



--[[

"Nanok! Good work today. The audience was joyful. I leave you here for the night, in your private den, full of toys, food, and costumes."
Nanok rested, luxurious in the satisfaction of a job well done. Truly, he was a circus bear who enjoyed his job. But sometimes, he thought . . . what . . . if he could be . . . something more?
"HEY! LISTEN!"
Nanok woke with surprise! Was someone talking to him?
"HEY! LISTEN! We need your help!"
Nanok regarded the winged orb with surprise. What could he be needed for, besides his standard cavorting and japery?
"HEY! LISTEN! The planet is under attack by aliens! You have to go shut down their reactor core!"
Nanok shook his head sadly, for he was merely a circus bear. What good could a circus bear do against fearsomely armed aliens, as such were nigh-guaranteed to be?
Nanok tried to explain his misgivings, though since he didn't speak English, this mostly came across as a loud barking noise.
"HEY! LISTEN! You're not even trying! You can turn your costumes into armor, and your plastic, spring-filled toys into shoes, and your water dish and food dish into blaster cannons!"
Nanok admitted (again, via barking) that this was an excellent and unarguable point, but those were things the Ringmaster had purchased for him. Could he risk the Ringmaster's wrath by ruining his riches for such a ridiculous reason?
"HEY! LISTEN! If the planet gets blown up, so will all your toys!"
Nanok acquiesced glumly, and pledged to convert the minimum of his possessions into combat equipment. He took stock of his gear. 

"HEY! LISTEN! Do you remember how to fire your blaster cannon? Or use your claws? Or walk? Or breathe?"
Nanok found this rather insulting. Very few circus bears ever forgot how to breathe. Statistically, it was insignificant.
"HEY! LISTEN! Use the arrow keys to move! Push Z to jump! Push C to swipe with your claws! Push X to fire your blaster cannon! You have a blaster cannon right? I wasn't paying attention 'cause I don't really care but I hope you do!"

"HEY! LISTEN! That wasn't very good at all! Let's get you some better equipment!"

Nanok found himself plunged into the center of the highly radioactive reactor core, but feared not, as circus bears are immune to radiation. 
"HEY! LISTEN! Good job! The world isn't ruined! I'll teleport you out of the reactor core immediately!"
Nanok questioned the sprite, as it seems that if he could be teleported out of a reactor core, he could have been teleported in as well.
"HEY! LISTEN! Shut up!"
Nanok watched as the sprite flitted away, leaving him alone in his den. Several minutes later, he heard a knocking on his door.

"Nanok! Your possessions! What have you done to them? They are all ruined, and somehow, radioactive! You are the worst circus bear, and you will have to work long hours to replace them."
Nanok was shamed, and admitted his fault.
"Nanok! Stop barking at me and go to work!"
CIRCUS BEAR STATUS: SHAMEFUL, YET TECHNICALLY, VICTORIOUS

"Nanok! Your possessions! Why are they damaged and modified and radioactive? I do not understand, but at least it did not affect all of them. Times will be tight, but we shall persevere. Ahoy!"
Nanok went back to work, with a heavy heart, but pride for his unsung heroism.
CIRCUS BEAR STATUS: ACCEPTABLY POWERFUL

"Nanok! How are you doing? Life is excellent! Oh, but look at this, this is radioactive. Nanok, why is this radioactive? What has happened? It shall have to be replaced. At least we are rich, thanks to your excellent performance. I shall return on the morrow."
Nanok rested in his comfortable den, knowing that he was, indeed, a bear to be reckoned with.
CIRCUS BEAR STATUS: MILDLY ABASHED, BUT BADASS

"Nanok! I have come bearing caviar and steak, to celebrate our wealth and our unsurprising, yet somehow notable, lack of radioactive toys. Feast with me, and on the morrow we vacation . . . to the moon!"
Nanok cheered, excited for his lunar adventure. Surely there would be no aliens on the moon . . . and that would be, somehow, thanks to him.
CIRCUS BEAR STATUS: EXCELSIOR!

]]
